import { AttributeType } from '../statistics.types';

// Check if a string is a valid date in the YYYY-MM-DD format
const isValidDate = (value: string): boolean => {
  const dateRegex = /^\d{4}-\d{2}-\d{2}$/; // Matches YYYY-MM-DD format
  const date = new Date(value);
  return (
    dateRegex.test(value) &&
    date.getFullYear() === Number(value.slice(0, 4)) &&
    date.getMonth() === Number(value.slice(5, 7)) - 1 &&
    date.getDate() === Number(value.slice(8, 10))
  );
};

// Check if a string is a valid time in the hh:mm:ss format
const isValidTime = (value: string): boolean => {
  const timeRegex = /^([01]\d|2[0-3]):([0-5]\d):([0-5]\d)$/;
  return timeRegex.test(value);
};

// Check if a string is a valid datetime in the YYYY-MM-DD hh:mm:ss format
const isValidDatetime = (value: string): boolean => {
  const datetimeRegex = /^\d{4}-\d{2}-\d{2} ([01]\d|2[0-3]):([0-5]\d):([0-5]\d)$/; // Correct regex
  const dateTimeParts = value.split(' ');
  const date = dateTimeParts[0];
  const time = dateTimeParts[1];

  return datetimeRegex.test(value) && isValidDate(date) && isValidTime(time);
};

// Check if a string is a valid number
const isValidNumber = (value: string): boolean => {
  return !isNaN(Number(value)) && !isNaN(parseFloat(value));
};

// Determines the type of an attribute
const getAttributeType = (value: any): AttributeType => {
  // Check for string representation of a number
  if (typeof value === 'string') {
    if (isValidNumber(value)) {
      return 'number';
    }
    if (isValidDatetime(value)) {
      return 'datetime';
    }
    if (isValidDate(value)) {
      return 'date';
    }
    if (isValidTime(value)) {
      return 'time';
    }
    return 'string';
  }

  if (typeof value === 'boolean') {
    return 'boolean';
  }

  if (typeof value === 'number') {
    return 'number';
  }

  if (Array.isArray(value)) {
    return 'array';
  }

  if (value instanceof Date) {
    return 'datetime'; // Assumes Date instance should be treated as datetime
  }
  // Check for object type
  if (typeof value === 'object' && value !== null) {
    return 'object';
  }

  return 'string'; // Default to string if uncertain
};

export { getAttributeType };
